package gui.run;

import gui.ClosableJFrame;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * Just like the RunTextFieldOld except that each
 * char causes an invocation of the run method
 */
public abstract class RunTextField extends
        JTextField
        implements ActionListener, KeyListener,
        Runnable {
    private String originalText = null;
    private boolean isNotRunWithKeyPressed = true;

    public RunTextField() {
        this("");
    }

    public RunTextField(int columns) {
        super(columns);
        initListeners();
    }

    public RunTextField(String text,
                        int columns) {
        super(text, columns);
        initListeners();
    }

    public RunTextField(javax.swing.text.Document doc,
                        String text,
                        int columns) {
        super(doc, text, columns);
        initListeners();
    }

    public RunTextField(int columns, boolean editable, boolean isRunWithKeyPressed) {
        this(columns);
        this.isNotRunWithKeyPressed = !isRunWithKeyPressed;
        initEditable(editable);
    }

    public RunTextField(String text, boolean editable, boolean isRunWithKeyPressed) {
        this(text);
        initEditable(editable);
        this.isNotRunWithKeyPressed = !isRunWithKeyPressed;

    }

    public RunTextField(boolean isRunWithKeyPressed) {
        this("");
        this.isNotRunWithKeyPressed = isRunWithKeyPressed;
    }

    public RunTextField(int columns, boolean editable) {
        this(columns);
        initEditable(editable);
    }

    public RunTextField(String text, boolean editable) {
        this(text);
        initEditable(editable);
    }


    public RunTextField(String text) {
        super(text);
        originalText = text;
        initListeners();
    }

    /**
     * Invoked when a component gains the keyboard focus.
     */
    public void focusGained(FocusEvent e) {
    }


    /**
     * Invoked when a component loses the keyboard focus.
     */
    public void focusLost(FocusEvent e) {
        run();
    }


    public String getOriginalText() {
        return originalText;
    }

    public void keyTyped(KeyEvent e) {

    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
        if (isNotRunWithKeyPressed)
            return;
        run();
    }


    private void initListeners() {
        addActionListener(this);
        ShortcutUtils.addShortcut(this);
        addKeyListener(this);
    }


    private void initEditable(boolean editable) {
        setEditable(editable);
        setFocusable(editable);
    }


    public void actionPerformed(java.awt.event.ActionEvent e) {
        run();
    }

    public static void main(String[] args) {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new FlowLayout());

        c.add(new RunTextField(20, true) {
            public void run() {
                System.out.println(getText());
            }
        });
        c.add(new RunCharField(20, true) {
            public void run() {
                System.out.println(getText());
            }
        });
        cf.setSize(200, 200);
        cf.setVisible(true);
    }

}